package com.ruoyi.system.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.ruoyi.system.model.salary.SalaryExcelModel;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;


/**
 * 2024/4/13
 */
public class TsExcelListener<T> extends AnalysisEventListener<T> {

    private final static Logger logger = LoggerFactory.getLogger(TsExcelListener.class);

    private int endRow;

    private int FINAL_FLAG = 0;

    List<SalaryExcelModel> salaryExcelModelList = new ArrayList<>();

    public final static Map<Integer, String> SOURCE_EXPENSES_MAP = new HashMap<>();

    static {
        SOURCE_EXPENSES_MAP.put(3, "基本生产原煤");
        SOURCE_EXPENSES_MAP.put(4, "基本生产洗煤");
        SOURCE_EXPENSES_MAP.put(5, "制造费原煤");
        SOURCE_EXPENSES_MAP.put(6, "制造费洗煤");
        SOURCE_EXPENSES_MAP.put(7, "劳保费用");
        SOURCE_EXPENSES_MAP.put(8, "管理费用");
        SOURCE_EXPENSES_MAP.put(9, "其他业务支出");
        SOURCE_EXPENSES_MAP.put(10, "研发费用");
    }

    @Override
    public void invoke(T t, AnalysisContext analysisContext) {
        SalaryExcelModel salaryExcelModel = (SalaryExcelModel) t;
        int flag = getSourceExpense(salaryExcelModel);
        salaryExcelModel.setSourceExpenses(SOURCE_EXPENSES_MAP.get(flag));
        //金属厂的费用来源填写金属厂
        if (StringUtils.contains(salaryExcelModel.getOrg(), "金属厂")) {
            salaryExcelModel.setSourceExpenses("金属厂");
        }
        if (StringUtils.containsAny(salaryExcelModel.getOrg(), "合计", "汇总")) {
            salaryExcelModel.setSourceExpenses(StringUtils.EMPTY);
        }
        if (StringUtils.equals("汇总合计", salaryExcelModel.getOrg())) {
            endRow = analysisContext.getCurrentRowNum();
        }
        int count = analysisContext.getCurrentRowNum();
        if (endRow == 0 || endRow == count) {
            salaryExcelModelList.add(salaryExcelModel);
        }
    }


    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        logger.info("解析完成!");
    }

    public List<SalaryExcelModel> getSalaryExcelModelList() {
        return salaryExcelModelList;
    }


    private int getSourceExpense(SalaryExcelModel salaryExcelModel) {
        String org = salaryExcelModel.getOrg();
        List<Integer> collect = SOURCE_EXPENSES_MAP.entrySet().stream().filter(v -> StringUtils.contains(org, v.getValue())).map(Map.Entry::getKey).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(collect)) {
            FINAL_FLAG = collect.get(0);
        }
        return FINAL_FLAG;
    }

}
