package com.ruoyi.system.service.impl;

import com.alibaba.excel.EasyExcel;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.system.constant.CoalConstant;
import com.ruoyi.system.listener.LxExcelListener;
import com.ruoyi.system.listener.LxInsuranceListener;
import com.ruoyi.system.mapper.SalaryMapper;
import com.ruoyi.system.model.salary.LxInsuranceModel;
import com.ruoyi.system.model.salary.LxSalaryModel;
import com.ruoyi.system.model.salary.SalaryExcelModel;
import com.ruoyi.system.service.LxSalaryService;
import com.ruoyi.system.support.LxSupport;
import lombok.SneakyThrows;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class LxSalaryServiceImpl implements LxSalaryService {

    private final SalaryMapper salaryMapper;

    public LxSalaryServiceImpl(SalaryMapper salaryMapper) {
        this.salaryMapper = salaryMapper;
    }

    @Override
    @SneakyThrows
    @Transactional
    public AjaxResult lxSave(MultipartFile multipartFile, String salaryDate) {
        String originalFilename = multipartFile.getOriginalFilename();
        InputStream inputStream = multipartFile.getInputStream();
        //传入工资表
        if (StringUtils.contains(originalFilename, "工资")) {
            LxExcelListener<LxSalaryModel> lxExcelListener = new LxExcelListener<>();
            EasyExcel.read(inputStream, LxSalaryModel.class, lxExcelListener).sheet(1).headRowNumber(2).doRead();
            List<LxSalaryModel> lxSalaryModelList = lxExcelListener.getLxSalaryModelList();
            List<SalaryExcelModel> salaryExcelModelList = LxSupport.getSalaryExcelModelList(lxSalaryModelList).stream().map(v -> {
                v.setBelongToCoal(CoalConstant.LX_MINE);
                v.setSalaryDate(salaryDate);
                v.setSourceExpenses(LxSupport.CHANNEL_MAP.get(v.getOrg()));
                return v;
            }).filter(v -> !StringUtils.equalsAny(v.getOrg(), "小计", "总计", "合计")).collect(Collectors.toList());
            //查询是否已经导入保险表
            List<SalaryExcelModel> list = salaryMapper.queryInsuranceSheet(salaryDate, CoalConstant.LX_MINE);
            if (CollectionUtils.isEmpty(list)) {
                salaryMapper.save(salaryExcelModelList);
            } else {
                salaryMapper.updateEmpty(StringUtils.EMPTY, salaryDate, CoalConstant.LX_MINE);
                List<SalaryExcelModel> salaryExcelModels = LxSupport.getSalaryExcelModels(list, salaryExcelModelList, salaryDate);
                salaryExcelModels.forEach(salaryMapper::updateInsurance);
            }
        }
        //传入社会保险缴费表
        if (StringUtils.contains(originalFilename, "保险")) {
            LxInsuranceListener<LxInsuranceModel> lxInsuranceListener = new LxInsuranceListener<>();
            EasyExcel.read(inputStream, LxInsuranceModel.class, lxInsuranceListener).sheet(0).headRowNumber(2).doRead();
            List<LxInsuranceModel> lxInsuranceModelList = lxInsuranceListener.getLxInsuranceModels();
            List<SalaryExcelModel> salaryExcelModelList = LxSupport.getInsuranceList(lxInsuranceModelList).stream().map(v -> {
                v.setSalaryDate(salaryDate);
                v.setBelongToCoal(CoalConstant.LX_MINE);
                v.setSourceExpenses(LxSupport.CHANNEL_MAP.get(v.getOrg()));
                return v;
            }).filter(v -> !StringUtils.equalsAny(v.getOrg(), "小计", "总计", "合计")).collect(Collectors.toList());
            //查询是否已经导入工资表
            List<SalaryExcelModel> list = salaryMapper.querySalarySheet(salaryDate, CoalConstant.LX_MINE);
            if (CollectionUtils.isEmpty(list)) {
                salaryMapper.save(salaryExcelModelList);
            } else {
                //重复上传先更新为空 再去插入
                salaryMapper.updateEmpty(StringUtils.EMPTY, salaryDate, CoalConstant.LX_MINE);
                List<SalaryExcelModel> salaryExcelModels = LxSupport.getSalaryExcelModels(salaryExcelModelList, list, salaryDate);
                salaryExcelModels.forEach(salaryMapper::updateInsurance);
            }
        }
        return AjaxResult.success("保存成功!");
    }


}
