package com.ruoyi.system.service.impl;


import com.alibaba.excel.EasyExcel;
import com.ruoyi.system.listener.ExcelListener;
import com.ruoyi.system.mapper.PowerWagesComMapper;
import com.ruoyi.system.mapper.PowerWagesDeMapper;
import com.ruoyi.system.model.power.dao.*;
import com.ruoyi.system.model.power.vo.PowerVo;
import com.ruoyi.system.service.PowerWagesService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author haiwe
 * @date 2024/6/26
 */
@Service
@RequiredArgsConstructor

public class PowerWagesServiceImpl implements PowerWagesService {
    private final PowerWagesComMapper ceMapper;
    private final PowerWagesDeMapper deMapper;


    /**
     * 导入详细表数据
     *
     * @param file
     * @param mineId
     * @param date
     * @return
     */
    @Override
    public Boolean readDExcel(MultipartFile file, String date, String mineId) {
        try {
            //定义需要解析的sheet名
            List<String> sheetList = Arrays.asList("电量数据专业化公司", "外界电量数据");
            // 定义需要过滤的关键词列表
            List<String> filterList = Arrays.asList("小计", "总表小计", "劳服总表", "校验", "null");
            List<String> filterList2 = Arrays.asList("20000", "30000", "40000");
            // 获取上传文件的输入流
            for (int i = 0; i < sheetList.size(); i++) {
                String type = "";
                switch (sheetList.get(i)) {
                    case "电量数据专业化公司":
                        type = "1";
                        break;
                    case "外界电量数据":
                        type = "2";
                        break;
                }
                PowerWagesComparation comparation = ceMapper.select(type, date, mineId);
                if (comparation != null) {
                    //获取对应表的id
                    Integer comparisonId = comparation.getComparisonId();
                    deMapper.deleteByDateAndMineIdAndComparisonId(mineId, date, comparisonId);
                    InputStream inputStream = file.getInputStream();
                    System.err.println(sheetList.get(i));
                    List<PowerWagesDetails> PowerList = EasyExcel
                            .read(inputStream, PowerWagesDetails.class,
                                    new ExcelListener<PowerWagesDetails>())
                            .sheet(sheetList.get(i))
                            .headRowNumber(3)
                            .doReadSync();
                    inputStream.close();

                    PowerList = PowerList.parallelStream()
                            .filter(v -> !filterList2.contains(v.getA2()))
                            .filter(v -> v.getA2() != null && !v.getA2().equals("null") && !filterList.contains(v.getA2()))
                            .filter(v -> v.getA3() != null && !v.getA3().equals("null") && !filterList.contains(v.getA3()))
                            .peek(v -> v.setComparisonId(comparisonId))
                            .collect(Collectors.toList());
                    deMapper.insertBatch(PowerList, mineId, date);
                }
            }
            return true;
        } catch (IOException e) {
            throw new RuntimeException("Excel文件读取失败：" + e.getMessage());
        }
    }

    @Override
    public Boolean readCExcel(MultipartFile file, String date, String mineId) {
        try {

            //定义需要解析的sheet名
            List<String> sheetList = Arrays.asList("电量数据专业化公司", "外界电量数据");
            // 获取上传文件的输入流
            for (int i = 0; i < sheetList.size(); i++) {
                String type = "";
                switch (sheetList.get(i)) {
                    case "电量数据专业化公司":
                        type = "1";
                        break;
                    case "外界电量数据":
                        type = "2";
                        break;
                }
                ceMapper.deleteByDateAndMineIdAndType(mineId, date, type);
                InputStream inputStream = file.getInputStream();
                System.err.println(sheetList.get(i));
                List<PowerWagesComparation> powerList = EasyExcel
                        .read(inputStream, PowerWagesComparation.class,
                                new ExcelListener<PowerWagesComparation>())
                        .sheet(sheetList.get(i))
                        .headRowNumber(2)
                        .doReadSync();
                inputStream.close();
                PowerWagesComparation comparation = powerList.get(0);

                comparation.setType(type);
                comparation.setMineId(mineId);
                comparation.setImportTime(date);
                ceMapper.insertBatch(comparation);
            }
            return true;
        } catch (IOException e) {
            throw new RuntimeException("Excel文件读取失败：" + e.getMessage());
        }
    }

    @Override
    public List<PowerVo> selectByMineIdAndDate(String mineId, String date) {
        //定义一个集合用来返回所需要的数据
        List<PowerVo> voList = new ArrayList<>();
        List<PowerWagesComparation> comparations = ceMapper.selectByMineIdAndDate(mineId, date);
        for (PowerWagesComparation comparation : comparations) {
            PowerVo powerVo = new PowerVo();
            List<PowerWagesDetails> details = deMapper.selectByCId(comparation.getComparisonId());
            powerVo.setDatas(comparation);
            powerVo.setDetails(details);
            voList.add(powerVo);
        }
        return voList;
    }

    /**
     * 特殊表表头处理
     *
     * @param powerDAO
     * @return
     */
    @Override
    public Boolean add(PowerDAO powerDAO) {
        //获取相关属性
        String type = powerDAO.getType();
        String importTime = powerDAO.getImportTime();
        String mineId = powerDAO.getMineId();

        List<Power> powerDetails = powerDAO.getPowerDetails();

        //删除表格中原有的数据
        ceMapper.deleteByDateAndMineIdAndType(mineId, importTime, type);

        Boolean added = ceMapper.add(powerDetails, mineId, importTime, type);
        if (!added) {
            return false;
        }
        return true;
    }

    @Override
    public Boolean pAdd(PowerAllDAO powerAllDAO) {
        //获取相关属性
        String type = powerAllDAO.getType();
        String importTime = powerAllDAO.getImportTime();
        String mineId = powerAllDAO.getMineId();
        List<Power> powerDetails = powerAllDAO.getPowerDetails();
        //查找对照表是否插入
        PowerWagesComparation comparation = ceMapper.select(type, importTime, mineId);
        if (comparation == null) {
            return false;
        }
        //获取对应表的id
        Integer comparisonId = comparation.getComparisonId();
        //删除原有数据
        deMapper.deleteByDateAndMineIdAndComparisonId(mineId, importTime, comparisonId);

        Boolean added = deMapper.add(powerDetails, comparisonId, importTime, mineId);
        if (!added) {
            return false;
        }
        return true;
    }

    /**
     * 林西矿基础数据导入
     * @param lxPowers 导入数据
     * @return true/false
     */
    @Override
    public Boolean insert(List<LxPower> lxPowers) {

        Stream<LxPower> lxPowerStream = lxPowers.stream();
        //去重
        Set<String> uniqueCombos = lxPowerStream
                .map(LxPower::getImportTime)
                .collect(Collectors.toSet());
        for (String uniqueCombo : uniqueCombos) {
            deMapper.deleteByDate(uniqueCombo);
        }

        Boolean inserted = deMapper.insert(lxPowers);
        if (!inserted) {
            return false;
        }
        return true;
    }
}






