package com.ruoyi.system.service.impl;


import com.ruoyi.common.core.domain.AjaxResult;


import com.ruoyi.system.mapper.SalaryMapper;
import com.ruoyi.system.model.newrule.NewRuleRequestModel;
import com.ruoyi.system.model.salary.SalaryExcelModel;
import com.ruoyi.system.model.salary.SalaryExpenseSourceModel;
import com.ruoyi.system.service.SalaryCurrentRuleService;
import com.ruoyi.system.support.CurrentRuleSalarySupport;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StopWatch;

import java.util.List;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class SalaryCurrentRuleServiceImpl implements SalaryCurrentRuleService {

    private final Logger logger = LoggerFactory.getLogger(SalaryCurrentRuleServiceImpl.class);

    private final SalaryMapper salaryMapper;

    public SalaryCurrentRuleServiceImpl(SalaryMapper salaryMapper) {
        this.salaryMapper = salaryMapper;
    }

    @Override
    public AjaxResult getCurrentRuleSalary(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        Optional<List<NewRuleRequestModel>> optional = Optional.ofNullable(newRuleSalaryRequestModelList);
        optional.ifPresent(v -> {
            String mineCode = newRuleSalaryRequestModelList.get(0).getMineid();
            String salaryDate = newRuleSalaryRequestModelList.get(0).getWageMonth();
            Function<List<NewRuleRequestModel>, List<SalaryExcelModel>> stringListFunction = CurrentRuleSalarySupport.MINE_MAP.get(mineCode);
            List<SalaryExcelModel> salaryExcelModels = stringListFunction.apply(newRuleSalaryRequestModelList);
            List<SalaryExpenseSourceModel> sourceModels = salaryMapper.getSources(mineCode);
            salaryExcelModels.forEach(salary -> sourceModels.forEach(source -> {
                if (StringUtils.equals(salary.getOrg(), source.getOrg())) {
                    salary.setSourceExpenses(source.getExpenseSource());
                }
            }));
            int count = salaryMapper.exist(mineCode, salaryDate);
            if (count > 0) {
                List<String> stringList = salaryMapper.selectByDateAndMineCode(salaryDate, mineCode);
                if (!CollectionUtils.isEmpty(stringList)) {
                    List<SalaryExcelModel> includeCollect = salaryExcelModels.stream().filter(value -> stringList.contains(value.getOrg())).collect(Collectors.toList());
                    if (!CollectionUtils.isEmpty(includeCollect)) {
                        includeCollect.forEach(salaryMapper::updateByDateAndProject);
                    }
                    List<SalaryExcelModel> noIncludeCollect = salaryExcelModels.stream().filter(value -> !stringList.contains(value.getOrg())).collect(Collectors.toList());
                    if (!CollectionUtils.isEmpty(noIncludeCollect)) {
                        salaryMapper.save(noIncludeCollect);
                    }
                }
            } else {
                salaryMapper.save(salaryExcelModels);
            }
        });
        stopWatch.stop();
        logger.info("组织:{}, 接口耗时:{}", newRuleSalaryRequestModelList.get(0).getMineid(), stopWatch.getTotalTimeMillis());
        return AjaxResult.success();
    }
}
