package com.ruoyi.system.service.impl;


import com.alibaba.excel.EasyExcelFactory;
import com.ruoyi.system.listener.ExcelListener;
import com.ruoyi.system.mapper.WagesCompilationMapper;
import com.ruoyi.system.mapper.WagesVersionMapper;
import com.ruoyi.system.model.wages.dao.WagesCompilationExcelDAO;
import com.ruoyi.system.service.WagesCompilationService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

/**
 * @author haiwe
 * @date 2024/6/7
 * 对照表
 */
@RequiredArgsConstructor
@Service
public class WagesCompilationServiceImpl implements WagesCompilationService {
    private final WagesCompilationMapper wagesCompilationMapper;
    private final WagesVersionMapper wagesVersionMapper;

    @Override
    @Transactional
    /**
     * 导入对照表及生产版本，只读sheet1，不支持同时导入多个版本
     */
    public void readExcel(MultipartFile file, String type, String mineId) {
        try {
            wagesVersionMapper.updateByWagesVersionToTypeAndMineId(type, mineId);
            wagesVersionMapper.insertWagesVersion(type, mineId);
            String vId = wagesVersionMapper.selectByWagesVersionToTypeAndMineId(type, mineId);
            InputStream inputStream = file.getInputStream();
            List<WagesCompilationExcelDAO> wagesList = EasyExcelFactory.read(inputStream, WagesCompilationExcelDAO.class, new ExcelListener<WagesCompilationExcelDAO>()).sheet(0).headRowNumber(1).doReadSync();
            inputStream.close();
            WagesCompilationExcelDAO wagesHeader = wagesList.get(0);
            wagesHeader.setMineId(mineId);
            wagesHeader.setType(type);
            wagesHeader.setVersionId(Integer.valueOf(vId));
            wagesCompilationMapper.insertBatch(wagesHeader);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public List<WagesCompilationExcelDAO> list() {
        return wagesCompilationMapper.list();
    }

}
