package com.ruoyi.system.support;


import com.alibaba.fastjson.JSON;
import com.ruoyi.system.constant.CoalConstant;
import com.ruoyi.system.mapper.SalaryMapper;
import com.ruoyi.system.model.newrule.NewRuleRequestModel;
import com.ruoyi.system.model.salary.ConfigModel;
import com.ruoyi.system.model.salary.SalaryExcelModel;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.ReflectionUtils;


import java.lang.reflect.Field;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Component
public class CurrentRuleSalarySupport {

    public final static Map<String, Function<List<NewRuleRequestModel>, List<SalaryExcelModel>>> MINE_MAP = new HashMap<>();

    static {
        MINE_MAP.put(CoalConstant.TS_MINE, CurrentRuleSalarySupport::getTsMineSalaryData);
        MINE_MAP.put(CoalConstant.LJT_MINE, CurrentRuleSalarySupport::getLjtMineSalaryData);
        MINE_MAP.put(CoalConstant.LX_MINE, CurrentRuleSalarySupport::getLxMineSalaryData);
        MINE_MAP.put(CoalConstant.DHT_MINE, CurrentRuleSalarySupport::getDhtMineSalaryData);
        MINE_MAP.put(CoalConstant.SH_MINE, CurrentRuleSalarySupport::getShMineSalaryData);
        MINE_MAP.put(CoalConstant.FK_MINE, CurrentRuleSalarySupport::getFKMineSalaryData);
        MINE_MAP.put(CoalConstant.HF_MINE, CurrentRuleSalarySupport::getHfMineSalaryData);
        MINE_MAP.put(CoalConstant.QJY_MINE, CurrentRuleSalarySupport::getQjyMineSalaryData);
        MINE_MAP.put(CoalConstant.YF_MINE, CurrentRuleSalarySupport::getYfMineSalaryData);
    }


    private static SalaryMapper salaryMapper;

    @Autowired
    public void setSalaryMapper(SalaryMapper salaryMapper) {
        CurrentRuleSalarySupport.salaryMapper = salaryMapper;
    }


    /**
     * 云飞
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getYfMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);
    }

    /**
     * 钱家营矿
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getQjyMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);
    }

    /**
     * 宏丰
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getHfMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);
    }

    /**
     * 范各庄矿
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getFKMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);
    }

    /**
     * 单侯
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getShMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);
    }


    /**
     * 唐山矿
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getTsMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);

    }

    /**
     * 吕家坨矿
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */

    private static List<SalaryExcelModel> getLjtMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);
    }

    /**
     * 林西矿
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getLxMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);
    }

    /**
     * 东欢坨矿
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getDhtMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);
    }


    public static List<SalaryExcelModel> getSalaryMapping(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        List<SalaryExcelModel> salaryExcelModelList = getSalaryExcelModelList(newRuleSalaryRequestModelList);
        List<SalaryExcelModel> finalList = new ArrayList<>();
        Map<String, List<SalaryExcelModel>> collect = salaryExcelModelList.stream().collect(Collectors.groupingBy(SalaryExcelModel::getOrg));
        collect.forEach((k, v) -> {
            SalaryExcelModel salaryExcelModel = mergeData(v);
            finalList.add(salaryExcelModel);
        });
        return finalList;
    }


    public static SalaryExcelModel mergeData(List<SalaryExcelModel> dataList) {
        return dataList.stream().collect(Collectors.collectingAndThen(Collectors.toMap(data -> data.getBelongToCoal() + data.getOrg() + data.getSalaryDate(), data -> data, (existingData, newData) -> {
            existingData.setPersonCount(
                    StringUtils.isEmpty(existingData.getPersonCount()) ? newData.getPersonCount() : existingData.getPersonCount());
            existingData.setMedicalInsurance(
                    StringUtils.isEmpty(existingData.getMedicalInsurance()) ? newData.getMedicalInsurance() : existingData.getMedicalInsurance());
            existingData.setOldInsurance(
                    StringUtils.isEmpty(existingData.getOldInsurance()) ? newData.getOldInsurance() : existingData.getOldInsurance());
            existingData.setUnemploymentInsurance
                    (StringUtils.isEmpty(existingData.getUnemploymentInsurance()) ? newData.getUnemploymentInsurance() : existingData.getUnemploymentInsurance());
            existingData.setSalary(
                    StringUtils.isEmpty(existingData.getSalary()) ? newData.getSalary() : existingData.getSalary());
            existingData.setMedicalSubsidy(
                    StringUtils.isEmpty(existingData.getMedicalSubsidy()) ? newData.getMedicalSubsidy() : existingData.getMedicalSubsidy());
            existingData.setSumSalary(
                    StringUtils.isEmpty(existingData.getSumSalary()) ? newData.getSumSalary() : existingData.getSumSalary());
            existingData.setAnnuity(
                    StringUtils.isEmpty(existingData.getAnnuity()) ? newData.getAnnuity() : existingData.getAnnuity());
            existingData.setSingleSupplyMedicalInsurance(
                    StringUtils.isEmpty(existingData.getSingleSupplyMedicalInsurance()) ? newData.getSingleSupplyMedicalInsurance() : existingData.getSingleSupplyMedicalInsurance());
            existingData.setSupplyMedicalInsurance(
                    StringUtils.isEmpty(existingData.getSupplyMedicalInsurance()) ? newData.getSupplyMedicalInsurance() : existingData.getSupplyMedicalInsurance());
            existingData.setBirthInsurance(
                    StringUtils.isEmpty(existingData.getBirthInsurance()) ? newData.getBirthInsurance() : existingData.getBirthInsurance());
            existingData.setInjuryInsurance(
                    StringUtils.isEmpty(existingData.getInjuryInsurance()) ? newData.getInjuryInsurance() : existingData.getInjuryInsurance());
            existingData.setCompanyHouseFund(
                    StringUtils.isEmpty(existingData.getCompanyHouseFund()) ? newData.getCompanyHouseFund() : existingData.getCompanyHouseFund());
            existingData.setHouseFund(
                    StringUtils.isEmpty(existingData.getHouseFund()) ? newData.getHouseFund() : existingData.getHouseFund());
            existingData.setLaborUnionDues(
                    StringUtils.isEmpty(existingData.getLaborUnionDues()) ? newData.getLaborUnionDues() : existingData.getLaborUnionDues());
            existingData.setEdufFees(
                    StringUtils.isEmpty(existingData.getEdufFees()) ? newData.getEdufFees() : existingData.getEdufFees());
            existingData.setOneChild(
                    StringUtils.isEmpty(existingData.getOneChild()) ? newData.getOneChild() : existingData.getOneChild());
            existingData.setLunch(
                    StringUtils.isEmpty(existingData.getLunch()) ? newData.getLunch() : existingData.getLunch());
            existingData.setLongAccountInsurance(
                    StringUtils.isEmpty(existingData.getLongAccountInsurance()) ? newData.getLongAccountInsurance() : existingData.getLongAccountInsurance());
            existingData.setHonorMoney(
                    StringUtils.isEmpty(existingData.getHonorMoney()) ? newData.getHonorMoney() : existingData.getHonorMoney());
            existingData.setWarmSubsidy(
                    StringUtils.isEmpty(existingData.getWarmSubsidy()) ? newData.getWarmSubsidy() : existingData.getWarmSubsidy());
            existingData.setHeatPreventSubsidy(
                    StringUtils.isEmpty(existingData.getHeatPreventSubsidy()) ? newData.getHeatPreventSubsidy() : existingData.getHeatPreventSubsidy());
            return existingData;
        }), map -> map.values().iterator().next()));
    }

    private static List<SalaryExcelModel> getSalaryExcelModelList(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        List<ConfigModel> configModelList = salaryMapper.getConfig();
        List<SalaryExcelModel> salaryExcelModelList = new ArrayList<>();
        Map<String, List<NewRuleRequestModel>> collect = newRuleSalaryRequestModelList.stream().collect(Collectors.groupingBy(NewRuleRequestModel::getRow));
        collect.forEach((key, value) -> {
            SalaryExcelModel salaryExcelModel = new SalaryExcelModel();
            for (NewRuleRequestModel newRuleRequestModel : value) {
                for (ConfigModel configModel : configModelList) {
                    if (StringUtils.contains(configModel.getColName(),newRuleRequestModel.getColoumn())){
                        Field field;
                        try {
                            field = SalaryExcelModel.class.getDeclaredField(configModel.getKeyName());
                            ReflectionUtils.makeAccessible(field);
                            ReflectionUtils.setField(field, salaryExcelModel, newRuleRequestModel.getValue());
                        } catch (NoSuchFieldException e) {
                            throw new RuntimeException(e);
                        }
                    }
                }
                salaryExcelModel.setBelongToCoal(newRuleRequestModel.getMineid());
                salaryExcelModel.setSalaryDate(newRuleRequestModel.getWageMonth());
            }
            salaryExcelModelList.add(salaryExcelModel);
        });
        return salaryExcelModelList;
    }

}
